/*
 * Initial main.c file generated by Glade. Edit as required.
 * Glade will not overwrite this file.
 *
 *  xfhell: An application to transmit and receive
 *  Feld Hell signals using a computer's sound card
 *
 *  Copyright (C) 2003  Neoklis Kyriazis
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details:
 *
 *  http://www.gnu.org/copyleft/gpl.txt
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "interface.h"
#include "support.h"
#include "xfhell.h"

/* xfhell main window */
GtkWidget *main_window = NULL;

/* Global widgets */
GtkWidget
  *gbl_drawingarea = NULL,
  *gbl_popup_menu  = NULL,
  *gbl_scope  = NULL,
  *gbl_deskew = NULL,
  *gbl_scope_label = NULL,
  *gbl_tx_scrolledwindow = NULL;

/* Receive window and waterfall pixbuffs */
extern pixbuf_data_t
  wfall_pixbuf,
  receive_pixbuf;

/* Text buffer for Tx text viewer */
GtkTextBuffer *gbl_tx_text_buffer = NULL;

/* Runtime config struct */
rc_data_t rc_data;

/* QSO record struct */
qso_record_t qso_record;

/* Signal samples buffer */
unsigned char *dsp_buff = NULL;

int
  contrast,    /* Receive text contrast req.*/
  rx_buff_idx; /* Audio samples bufer index */

char
  label[9][LABEL_LENGTH], /* Labels for F1-F9 commands */
  *macro[9];              /* Macros attached to F1-F9  */

/* Signal handler */
static void sig_handler( int signal );

/*------------------------------------------------------------------------*/

  int
main (int argc, char *argv[])
{
  /* Command line option returned by getopt() */
  int option;

  /* getopt() variables */
  extern char *optarg;
  extern int optind, opterr, optopt;

  /* New and old actions for sigaction() */
  struct sigaction sa_new, sa_old;


  /* Initialize new actions */
  sa_new.sa_handler = sig_handler;
  sigemptyset( &sa_new.sa_mask );
  sa_new.sa_flags = 0;

  /* Register function to handle signals */
  sigaction( SIGINT,  &sa_new, &sa_old );
  sigaction( SIGSEGV, &sa_new, 0 );
  sigaction( SIGFPE,  &sa_new, 0 );
  sigaction( SIGTERM, &sa_new, 0 );
  sigaction( SIGABRT, &sa_new, 0 );

  gtk_set_locale ();
  gtk_init (&argc, &argv);

  /* Process command line options */
  while( (option = getopt(argc, argv, "hv") ) != -1 )
	switch( option )
	{
	  case 'h': /* Print usage and exit */
		Usage();
		exit( 0 );

	  case 'v': /* Print version */
		fprintf( stderr, "%s %s\n", PACKAGE, VERSION );
		exit( 0 );

	  default: /* Print usage and exit */
		Usage();
		exit( -1 );

	} /* End of switch( option ) */

  /* Null pixbuf pointers */
  receive_pixbuf.pixbuf = NULL;
  wfall_pixbuf.pixbuf   = NULL;

  /*
   * The following code was added by Glade to create one of each component
   * (except popup menus), just so that you see something after building
   * the project. Delete any components that you don't want shown initially.
   */
  main_window    = create_main_window ();
  gbl_popup_menu = create_main_menu();

  /* Get drawing area widget */
  gbl_drawingarea = lookup_widget( main_window, "drawingarea" );

  /* Get Tx text buffer and scroller */
  gbl_tx_text_buffer = gtk_text_view_get_buffer
	( GTK_TEXT_VIEW(lookup_widget(main_window, "tx_textview")) );
  gbl_tx_scrolledwindow = lookup_widget( main_window, "tx_scrolledwindow" );

  /* Get scope widget */
  gbl_scope = lookup_widget( main_window, "scope" );
  gbl_scope_label = lookup_widget( main_window, "label25" );

  /* Get and enter contrast value */
  contrast = ELEMENT_MAX - (ELEMENT_MAX/10) *
	 (int)( gtk_range_get_adjustment(GTK_RANGE(
			 lookup_widget(main_window, "contrast")))->value );

  /* Set labels to Xmit and Rcve buttons */
  {
	GtkWidget
	  *button,
	  *xmit_status,
	  *rcve_status;

	button = lookup_widget( main_window, "xmit" );
	xmit_status = gtk_label_new ("TRANSMIT");
	gtk_widget_show (xmit_status);
	gtk_container_add (GTK_CONTAINER (button), xmit_status);
	g_object_set_data_full (G_OBJECT (main_window), "xmit_status",
		gtk_widget_ref (xmit_status),(GDestroyNotify) gtk_widget_unref);

	button = lookup_widget( main_window, "rcve" );
	rcve_status = gtk_label_new ("RECEIVE");
	gtk_widget_show (rcve_status);
	gtk_container_add (GTK_CONTAINER (button), rcve_status);
	g_object_set_data_full (G_OBJECT (main_window), "rcve_status",
		gtk_widget_ref (rcve_status),(GDestroyNotify) gtk_widget_unref);
  }

  /* Set default flags */
  SetFlag( TX2RX_LOOPBACK );
  SetFlag( RECORD_QSO );
  SetFlag( SAVE_PIXBUF );
  SetFlag( CAPITALIZE );
  rc_data.num_rows  = 2;
  rc_data.rev_video = FALSE;

  /* Initialize fft */
  Ifft_Init();

  /* Show main window */
  gtk_widget_show (main_window);

  /* Load runtime config file, abort on error */
  g_idle_add( Load_Config, main_window );

  gtk_main ();

  return 0;
}

/*-------------------------------------------------------------------*/

/*  Load_Config()
 *
 *  Loads the xfhellrc configuration file
 */

  gboolean
Load_Config( gpointer data )
{
  /* For setting macro button labels */
  GtkWidget *button;
  char bname[7];

  int
	lb_idx, /* Label buffer index */
	mc_idx, /* Macro buffer index */
	ln_idx, /* Line buffer index  */
	ln_len, /* Line buffer length */
	tg_cnt, /* Count '<' and '>'  */
	height, /* Set Receive window height */
	idx;

  char
	rc_fpath[64], /* File path to xfhellrc */
	line[81];     /* Buffer for Load_Line  */

  /* Config file pointer */
  FILE *xfhellrc;


  /* Setup file path to xfhellrc */
  snprintf( rc_fpath, 64, "%s/xfhell/xfhellrc", getenv("HOME") );

  /* Open xfhellrc file */
  xfhellrc = fopen( rc_fpath, "r" );
  if( xfhellrc == NULL )
  {
	perror( rc_fpath );
	Error_Dialog(
		"Failed to open xfhellrc file\n"
		"Quit xfhell and correct", TRUE );
	return( FALSE );
  }

  /*** Read runtime configuration data ***/
  /* Read mixer device, abort if EOF */
  if( Load_Line(line, xfhellrc, "Mixer Device" ) != SUCCESS )
	return( FALSE );
  strncpy( rc_data.mixer, line, 14 );
  rc_data.mixer[14] = '\0';

  /* Read input DSP device, abort if EOF */
  if( Load_Line(line, xfhellrc, "Input DSP Device" ) != SUCCESS )
	return( FALSE );
  strncpy( rc_data.dsp_in, line, 14 );
  rc_data.dsp_in[14] = '\0';

  /* Read output DSP device, abort if EOF */
  if( Load_Line(line, xfhellrc, "Output DSP Device" ) != SUCCESS )
	return( FALSE );
  strncpy( rc_data.dsp_out, line, 14 );
  rc_data.dsp_out[14] = '\0';

  /* Read DSP speed Samples/sec,  abort if EOF */
  if( Load_Line(line, xfhellrc, "DSP Speed" ) != SUCCESS )
	return( FALSE );
  rc_data.dsp_speed = atoi( line );

  /* Read recording source, abort if EOF */
  if( Load_Line(line, xfhellrc, "Recording Source" ) != SUCCESS )
	return( FALSE );
  strncpy( rc_data.rec_src, line, 9 );
  rc_data.rec_src[9] = '\0';

  /* Read Recording level device, abort if EOF */
  if( Load_Line(line, xfhellrc, "Recording Level Device" ) != SUCCESS )
	return( FALSE );
  strncpy( rc_data.lev_dev, line, 9 );
  rc_data.lev_dev[9] = '\0';

  /* Read Recording level, abort if EOF */
  if( Load_Line(line, xfhellrc, "Recording Level" ) != SUCCESS )
	return( FALSE );
  rc_data.rec_lev = atoi( line );

  /* Read Volume level, abort if EOF */
  if( Load_Line(line, xfhellrc, "Volume Level" ) != SUCCESS )
	return( FALSE );
  rc_data.vol_lev = atoi( line );

  /* Read mode, abort if EOF */
  if( Load_Line(line, xfhellrc, "STEREO|MONO Mode" ) != SUCCESS )
	return( FALSE );
  strncpy( rc_data.mode, line, 6 );
  rc_data.mode[6] = '\0';

  /* Read channel used, abort if EOF */
  if( Load_Line(line, xfhellrc, "LEFT|RIGHT Channel" ) != SUCCESS )
	return( FALSE );
  strncpy( rc_data.channel, line, 5 );
  rc_data.channel[5] = '\0';

  /* Read CW encoder speed, abort if EOF */
  if( Load_Line(line, xfhellrc, "CW Encoder Speed" ) != SUCCESS )
	return( FALSE );
  rc_data.wpm = atoi( line );

  /* Read CW message to send at tag '*', abort if EOF */
  if( Load_Line(line, xfhellrc, "CW Message" ) != SUCCESS )
	return( FALSE );
  strncpy( rc_data.cw_msg, line, 24 );
  rc_data.cw_msg[24] = '\0';

  /* Read word wrap column, abort if EOF */
  if( Load_Line(line, xfhellrc, "Word Wrap Column" ) != SUCCESS )
	return( FALSE );
  rc_data.wwrap = atoi( line );

  /*** Setup default main menu items ***/
  /* Read FT847 serial port device, abort if EOF */
  if( Load_Line(line, xfhellrc, "FT847 Serial Port" ) != SUCCESS )
	return( FALSE );
  strncpy( rc_data.serial1, line, 14 );
  rc_data.serial1[14] = '\0';

  /* Read FT847 CAT enable flag, abort if EOF */
  if( Load_Line(line, xfhellrc, "FT847 CAT Enable" ) != SUCCESS )
	return( FALSE );
  if( strcmp(line, "yes") == 0 )
	SetFlag( ENABLE_CAT_847 );
  else
	if( strcmp(line, "no") == 0 )
	  ClearFlag( ENABLE_CAT_847 );
	else
	{
	  Error_Dialog(
		  "Error reading xfhellrc\n"
		  "Unrecognized FT-847 CAT option\n"
		  "Quit xfhell and correct", TRUE );
	  return( FALSE );
	}

  /* Read FT857 serial port device, abort if EOF */
  if( Load_Line(line, xfhellrc, "FT857 Serial Port" ) != SUCCESS )
	return( FALSE );
  strncpy( rc_data.serial2, line, 14 );
  rc_data.serial2[14] = '\0';

  /* Read FT857 CAT enable flag, abort if EOF */
  if( Load_Line(line, xfhellrc, "FT857 CAT Enable" ) != SUCCESS )
	return( FALSE );
  if( strcmp(line, "yes") == 0 )
	SetFlag( ENABLE_CAT_857 );
  else
	if( strcmp(line, "no") == 0 )
	  ClearFlag( ENABLE_CAT_857 );
	else
	{
	  Error_Dialog(
		  "Error reading xfhellrc\n"
		  "Unrecognized FT-857 CAT option\n"
		  "Quit xfhell and correct", TRUE );
	  return( FALSE );
	}

  /* Try to auto-detect transceiver */
  if( Load_Line(line, xfhellrc, "CAT Auto Detect" ) != SUCCESS )
	return( FALSE );
  if( strcmp(line, "yes") == 0 )
	SetFlag( ENABLE_CAT );
  else
	if( strcmp(line, "no") == 0 )
	  ClearFlag( ENABLE_CAT );
	else
	{
	  Error_Dialog(
		  "Error reading xfhellrc\n"
		  "Unrecognized CAT autodetect option\n"
		  "Quit xfhell and correct", TRUE );
	  return( FALSE );
	}

  /* Read RTS-Control serial port device, abort if EOF */
  if( Load_Line(line, xfhellrc, "RTS-Control Serial Port" ) != SUCCESS )
	return( FALSE );
  strncpy( rc_data.serial3, line, 14 );
  rc_data.serial3[14] = '\0';

  /* Read RTS-Control enable flag, abort if EOF */
  if( Load_Line(line, xfhellrc, "RTS-Control Enable" ) != SUCCESS )
	return( FALSE );
  if( strcmp(line, "yes") == 0 )
	SetFlag( ENABLE_RTS_CTL );
  else
	if( strcmp(line, "no") == 0 )
	  ClearFlag( ENABLE_RTS_CTL );
	else
	{
	  Error_Dialog(
		  "Error reading xfhellrc\n"
		  "Unrecognized RTS-Control option\n"
		  "Quit xfhell and correct", TRUE );
	  return( FALSE );
	}

  /* Read Record QSO enable flag, abort if EOF */
  if( Load_Line(line, xfhellrc, "Record QSOs Enable" ) != SUCCESS )
	return( FALSE );

  if( strcmp(line, "yes") == 0 )
  {
	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(
		  lookup_widget(gbl_popup_menu, "record_qsos")), TRUE );
	SetFlag( RECORD_QSO );
  }
  else
	if( strcmp(line, "no") == 0 )
	  gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(
			lookup_widget(gbl_popup_menu, "record_qsos")), FALSE );
	else
	{
	  Error_Dialog(
		  "Error reading xfhellrc\n"
		  "Unrecognized Record QSO option\n"
		  "Quit xfhell and correct", TRUE );
	  return( FALSE );
	}

  /* Disable callbacks to menu handlers to prevent messups */
  SetFlag( NO_CALLBACKS );

  /* Read Hellschreiber mode */
  if( Load_Line(line, xfhellrc, "Hellschreiber Mode") != SUCCESS )
	return( FALSE );
  if( strcmp(line, "FeldHell") == 0 )
  {
	/* Set appropriate radio button active */
	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(
		  lookup_widget(gbl_popup_menu, "feldhell")), TRUE);
	SetFlag( MODE_FELDHELL );
  }
  else
	if( strcmp(line, "FMHell") == 0 )
	{
	  /* Set appropriate radio button active */
	  gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(
			lookup_widget(gbl_popup_menu, "fmhell")), TRUE);
	  ClearFlag( MODE_FELDHELL );
	}
	else
	{
	  Error_Dialog(
		  "Error reading xfhellrc\n"
		  "Unrecognized Hell Mode option\n"
		  "Quit xfhell and correct", TRUE );
	  return( FALSE );
	}

  /* Read Hellschreiber baud rate */
  if( Load_Line(line, xfhellrc, "Hellschreiber Baud Rate") != SUCCESS )
	return( FALSE );
  rc_data.baud_rate = atof(line);

  /* Read dot size */
  if( Load_Line(line, xfhellrc, "Read Dot Size") != SUCCESS )
	return( FALSE );
  rc_data.dot_hgt = atoi( line );
  if( (rc_data.dot_hgt > 4) || (rc_data.dot_hgt < 2) )
  {
	Error_Dialog(
		"Error reading xfhellrc\n"
		"Dot size incorrect\n"
		"Quit xfhell and correct", TRUE );
	return( FALSE );
  }
  rc_data.dot_wdt  = (rc_data.dot_hgt+1)/2;

  /* Read Capitalize enable flag, abort if EOF */
  if( Load_Line(line, xfhellrc, "Capitalize Enable" ) != SUCCESS )
	return( FALSE );
  if( strcmp(line, "yes") == 0 )
  {
	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(
		  lookup_widget(gbl_popup_menu, "capitalize_letters")), TRUE );
	SetFlag( CAPITALIZE );
  }
  else
	if( strcmp(line, "no") == 0 )
	  gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(
			lookup_widget(gbl_popup_menu, "capitalize_letters")), FALSE );
	else
	{
	  Error_Dialog(
		  "Error reading xfhellrc\n"
		  "Unrecognized Capitalize option\n"
		  "Quit xfhell and correct", TRUE );
	  return( FALSE );
	}

  /* Read input tone (Rx BFO) frequency */
  if( Load_Line(line, xfhellrc, "Input Tone") != SUCCESS )
	return( FALSE );
  rc_data.tone_freq = atoi( line );
  rc_data.red_line = FFT_SIZE / 4;

  /* Read Receive window half size setting */
  if( Load_Line(line, xfhellrc, "Receive Window Size") != SUCCESS )
	return( FALSE );
  if( strcmp(line, "no") == 0 ) height = 0;
  else
	if( strcmp(line, "yes") == 0 ) height = 1;
	else
	{
	  Error_Dialog(
		  "Error reading xfhellrc\n"
		  "Unrecognized Receive Window Size option\n"
		  "Quit xfhell and correct", TRUE );
	  return( FALSE );
	}

  /* Read operator callsign, abort if EOF */
  if( Load_Line(line, xfhellrc, "Operator Callsign" ) != SUCCESS )
	return( FALSE );
  strncpy( rc_data.call, line, 14 );
  rc_data.call[14] = '\0';

  /* Read operator name, abort if EOF */
  if( Load_Line(line, xfhellrc, "Operator Name" ) != SUCCESS )
	return( FALSE );
  strncpy( rc_data.name, line, 12 );
  rc_data.name[12] = '\0';

  /* Read Zone, abort if EOF */
  if( Load_Line(line, xfhellrc, "ITU Zone" ) != SUCCESS )
	return( FALSE );
  strncpy( rc_data.zone, line, 14 );
  rc_data.zone[14] = '\0';

  /* Read QTH, abort if EOF */
  if( Load_Line(line, xfhellrc, "QTH Name" ) != SUCCESS )
	return( FALSE );
  strncpy( rc_data.qth, line, 12 );
  rc_data.qth[12] = '\0';

  /* Read QTH locator, abort if EOF */
  if( Load_Line(line, xfhellrc, "QTH Locator" ) != SUCCESS )
	return( FALSE );
  strncpy( rc_data.loc, line, 6 );
  rc_data.loc[6] = '\0';

  /* Read Transmitter name, abort if EOF */
  if( Load_Line(line, xfhellrc, "Transmitter Name" ) != SUCCESS )
	return( FALSE );
  strncpy( rc_data.tx, line, 11 );
  rc_data.tx[11] = '\0';

  /* Read Transmitter power, abort if EOF */
  if( Load_Line(line, xfhellrc, "Transmitter Power" ) != SUCCESS )
	return( FALSE );
  strncpy( rc_data.tx_power, line, 4 );
  rc_data.tx_power[4] = '\0';

  /* Read Transmitter antenna, abort if EOF */
  if( Load_Line(line, xfhellrc, "Transmitter Antenna" ) != SUCCESS )
	return( FALSE );
  strncpy( rc_data.tx_ant, line, 15 );
  rc_data.tx_ant[15] = '\0';

  /* Read Receiver name, abort if EOF */
  if( Load_Line(line, xfhellrc, "Receiver Name" ) != SUCCESS )
	return( FALSE );
  strncpy( rc_data.rx, line, 11 );
  rc_data.rx[11] = '\0';

  /* Read Receiver N.Fig, abort if EOF */
  if( Load_Line(line, xfhellrc, "Receiver Noise Figure" ) != SUCCESS )
	return( FALSE );
  strncpy( rc_data.rx_nfig, line, 4 );
  rc_data.rx_nfig[4] = '\0';

  /* Read Receiver antenna, abort if EOF */
  if( Load_Line(line, xfhellrc, "Receiver Antenna" ) != SUCCESS )
	return( FALSE );
  strncpy( rc_data.rx_ant, line, 15 );
  rc_data.rx_ant[15] = '\0';

  /* Read propagation medium */
  if( Load_Line(line, xfhellrc, "Propagation Medium" ) != SUCCESS )
	return( FALSE );
  strncpy( qso_record.via, line, 12 );
  qso_record.via[12] = '\0';

  /* Read default remarks on QSO */
  if( Load_Line(line, xfhellrc, "Default Remarks" ) != SUCCESS )
	return( FALSE );
  strncpy( qso_record.remarks, line, 45 );
  qso_record.remarks[45] = '\0';

  /*** Read macro labels from rc file ***/
  /* Null buffer pointers just in case */
  for( idx = 0; idx < NUM_OF_LABELS; idx++ )
	macro[idx] = NULL;

  /* Preload a line */
  if( Load_Line(line, xfhellrc, "Macro Labels" ) == SUCCESS )
	for( lb_idx = 0; lb_idx < NUM_OF_LABELS; lb_idx++ )
	{
	  /* Abort if first line is not a label */
	  if( line[0] != '[' )
	  {
		fprintf( stderr, "xfhell: First Macro line not a label\n%s\n", line );
		Error_Dialog(
			"Error reading xfhellrc\n"
			"First Macro line not a label\n"
			"Quit xfhell and correct", TRUE);
		return( FALSE );
	  }

	  /* Look for closing ']' */
	  ln_idx = 1;
	  while( (ln_idx <= LABEL_LENGTH) && (line[ln_idx] != ']') )
		ln_idx++;

	  /* Abort if label string > LABEL_LENGTH char + [] */
	  if( ln_idx > LABEL_LENGTH )
	  {
		fprintf( stderr, "xfhell: Macro label too long\n%s\n", line );
		Error_Dialog(
			"Error reading xfhellrc\n"
			"Macro label too long\n"
			"Quit xfhell and correct", TRUE);
		return( FALSE );
	  }

	  /* Enter label string if all OK */
	  snprintf( label[lb_idx], LABEL_LENGTH, "(F%d)", lb_idx+1 );
	  line[strlen(line)-1] = '\0';
	  strncat( label[lb_idx], &line[1], LABEL_LENGTH );

	  /* Set macro button labels */
	  snprintf( bname, 7, "macro%d", lb_idx+1 );
	  button = lookup_widget( main_window, bname );
	  gtk_button_set_label( GTK_BUTTON(button), label[lb_idx] );

	  /*** Read macros ***/
	  mc_idx = tg_cnt = 0;
	  do
	  {
		if( Load_Line(line, xfhellrc, "Macros") != SUCCESS )
		  return( FALSE );;

		/* Read labels if line starts with '[' */
		if( line[0] == '[' ) break;

		/* Allocate memory to macro buffer */
		ln_len = strlen( line );

		/* Ignore | end-of-macro delimeter */
		if( line[ln_len-1] == '|' ) ln_len--;
		macro[lb_idx] = realloc(macro[lb_idx], ln_len + mc_idx + 2);

		/* Concatenate lines to macro */
		for( ln_idx = 0; ln_idx < ln_len; ln_idx++ )
		{
		  macro[lb_idx][mc_idx++] = line[ln_idx];

		  /* Count tag delimiters */
		  if( (line[ln_idx] == '<') ||
			  (line[ln_idx] == '>') )
			tg_cnt++;
		}

		/* Terminate Macro with LF and null */
		macro[lb_idx][mc_idx++] = LF;
		macro[lb_idx][mc_idx] = '\0';
	  } /* do */
	  while( line[ln_len] != '|'  );

	  /* Abort if tag delimiters are odd number */
	  if( tg_cnt & 1 )
	  {
		fprintf( stderr, "xfhell: Malformed tag in Macro\n%s\n", line );
		Error_Dialog(
			"Error reading xfhellrc\n"
			"Malformed tag in Macro\n"
			"Quit xfhell and correct", TRUE );
		return( FALSE );
	  }

	} /* for( lb_idx = 0; lb_idx < NUM_OF_LABELS; lb_idx++ ) */

  /*** Do the global config variables ***/
  /* Set stereo or mono mode */
  if( strcmp(rc_data.mode, "STEREO") == 0 )
	rc_data.num_chan = 2;
  else
	if( strcmp(rc_data.mode, "MONO") == 0 )
	  rc_data.num_chan = 1;
	else
	{
	  Error_Dialog(
		  "Error reading xfhellrc file\n"
		  "Sound card mode invalid\n"
		  "Quit xfhell and correct", TRUE  );
	  return( FALSE );
	}

  /* Channel in use left or right */
  if( strcmp(rc_data.channel, "RIGHT") == 0 )
	rc_data.chan_used = 1;
  else
	if( strcmp(rc_data.channel, "LEFT") == 0 )
	  rc_data.chan_used = 0;
	else
	{
	  Error_Dialog(
		  "Error reading xfhellrc file\n"
		  "Channel in use invalid\n"
		  "Quit xfhell and correct", TRUE  );
	  return( FALSE );
	}

  /* Size of samples buffer depends on stereo/mono mode */
  rc_data.rxbuff_len = RX_BUFFER_SIZE * rc_data.num_chan;
  /* Index to rx samples buffer (set to end) */
  rx_buff_idx  = rc_data.rxbuff_len;

  /* Set appropriate baud rate menu item */
  {
	char btn[6];
	GtkWidget *item;

	snprintf( btn, 6, "br%d", (int)rc_data.baud_rate );
	item = lookup_widget(gbl_popup_menu, btn);
	if( !item )
	{
	  Error_Dialog(
		  "Error reading xfhellrc\n"
		  "Invalid Baud rate\n"
		  "Quit xfhell and correct", TRUE );
	  return( FALSE );
	}
	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(item), TRUE );
  }

  /* Set apropriate dot height menu item */
  switch( rc_data.dot_hgt )
  {
	case 4:
	  gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(
			lookup_widget(gbl_popup_menu, "md2x4")), TRUE);
	  break;

	case 2:
	  gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(
			lookup_widget(gbl_popup_menu, "md1x2")), TRUE);

	default:
	  Error_Dialog(
		  "Error reading xfhellrc\n"
		  "Invalid dot height\n"
		  "Quit xfhell and correct", TRUE );
	  return( FALSE );
  }

  /* Set up CAT if enabled */
  if( isFlagSet(ENABLE_CAT_847) )
	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(
		  lookup_widget(gbl_popup_menu, "ft847")), TRUE );
  else
	if( isFlagSet(ENABLE_CAT_857) )
	  gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(
			lookup_widget(gbl_popup_menu, "ft857")), TRUE );
	else
	  if( isFlagSet(ENABLE_CAT) ) /* Auto-detect transceiver */
	  {
		ClearFlag( ENABLE_CAT );
		Close_Tcvr_Serial();
		ClearFlag( ENABLE_CAT_857 | SERIAL_IO_ERROR );
		SetFlag( ENABLE_CAT_847 );
		Open_Tcvr_Serial();

		if( isFlagSet(ENABLE_CAT) )
		  gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(
				lookup_widget(gbl_popup_menu, "ft847")), TRUE );
		else
		{
		  ClearFlag( ENABLE_CAT_847 | SERIAL_IO_ERROR );
		  SetFlag( ENABLE_CAT_857 );
		  Open_Tcvr_Serial();
		  if( isFlagSet(ENABLE_CAT) )
			gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(
				  lookup_widget(gbl_popup_menu, "ft857")), TRUE );
		  else
		  {
			ClearFlag( ENABLE_CAT_857 | SERIAL_IO_ERROR );
			gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(
				  lookup_widget(gbl_popup_menu, "none")), TRUE );
		  }
		} /* isFlagSet(ENABLE_CAT) else */
	  } /* if( isFlagSet(ENABLE_CAT) */
	  else
		if( isFlagSet(ENABLE_RTS_CTL) )
		  gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(
				lookup_widget(gbl_popup_menu, "rts_ctl")), TRUE );

  /* Wait for GTK to complete its tasks */
  while( g_main_context_iteration(NULL, FALSE) );

  /* Set Receive window height check button */
  if( !height )
  {
	GtkToggleButton *toggle = GTK_TOGGLE_BUTTON(
		lookup_widget(main_window, "height_checkbutton"));
	gtk_toggle_button_set_active( toggle, FALSE );
  }

  /* Close rc file */
  fclose( xfhellrc );

  /* Enable menu handlers */
  ClearFlag( NO_CALLBACKS );

  /* Clear Font file name and set params */
  rc_data.font_file[0] = '\0';
  New_Parameters();

  return( FALSE );

} /* End of Load_Config() */

/*------------------------------------------------------------------*/

/*  Load_Line()
 *
 *  Loads a line from a file, aborts on failure. Lines beginning
 *  with a '#' are ignored as comments. At the end of file EOF is
 *  returned. Lines assumed maximum 80 characters long.
 */

  int
Load_Line( char *buff, FILE *pfile, char *messg )
{
  int
	num_chr, /* Number of characters read, excluding lf/cr */
	chr;     /* Character read by getc() */
  char err_msg[128];

  /* Prepare error message */
  snprintf( err_msg, 128,
	  "Error reading %s\n"
	  "Premature EOF (End Of File)", messg );

  /* Clear buffer at start */
  buff[0] = '\0';
  num_chr = 0;

  /* Get next character, return error if chr = EOF */
  if( (chr = fgetc(pfile)) == EOF )
  {
	fprintf( stderr, "xfhell: %s\n", err_msg );
	Error_Dialog( err_msg, TRUE );
	return( EOF );
  }

  /* Ignore commented lines, white spaces and eol/cr */
  while(
	  (chr == '#') ||
	  (chr == HT ) ||
	  (chr == ' ') ||
	  (chr == CR ) ||
	  (chr == LF ) )
  {
	/* Go to the end of line (look for LF or CR) */
	while( (chr != CR) && (chr != LF) )
	  /* Get next character, return error if chr = EOF */
	  if( (chr = fgetc(pfile)) == EOF )
	  {
		fprintf( stderr, "xfhell: %s\n", err_msg );
		Error_Dialog( err_msg, TRUE );
		return( EOF );
	  }

	/* Dump any CR/LF remaining */
	while( (chr == CR) || (chr == LF) )
	  /* Get next character, return error if chr = EOF */
	  if( (chr = fgetc(pfile)) == EOF )
	  {
		fprintf( stderr, "xfhell: %s\n", err_msg );
		Error_Dialog( err_msg, TRUE );
		return( EOF );
	  }

  } /* End of while( (chr == '#') || ... */

  /* Continue reading characters from file till
   * number of characters = 80 or EOF or CR/LF */
  while( num_chr < 80 )
  {
	/* If LF/CR reached before filling buffer, return line */
	if( (chr == LF) || (chr == CR) ) break;

	/* Enter new character to line buffer */
	buff[num_chr++] = chr;

	/* Get next character */
	if( (chr = fgetc(pfile)) == EOF )
	{
	  /* Terminate buffer as a string if chr = EOF */
	  buff[num_chr] = '\0';
	  return( SUCCESS );
	}

	/* Abort if end of line not reached at 80 char. */
	if( (num_chr == 80) && (chr != LF) && (chr != CR) )
	{
	  /* Terminate buffer as a string */
	  buff[num_chr] = '\0';
	  snprintf( err_msg, 128,
		  "Error reading %s\n"
		  "Line longer than 80 characters", messg );
	  fprintf( stderr, "xfhell: %s\n%s\n", err_msg, buff );
	  Error_Dialog( err_msg, TRUE );
	  return( ERROR );
	}

  } /* End of while( num_chr < max_chr ) */

  /* Terminate buffer as a string */
  buff[num_chr] = '\0';

  return( SUCCESS );

} /* End of Load_Line() */

/*------------------------------------------------------------------------*/

/*  Usage()
 *
 *  Prints usage information
 */

  void
Usage( void )
{
  fprintf( stderr, "%s\n",
	  "Usage: xfhell [-hv]" );

  fprintf( stderr, "%s\n",
	  "       -h: Print this usage information and exit");

  fprintf( stderr, "%s\n",
	  "       -v: Print version number and exit");

} /* End of Usage() */

/*------------------------------------------------------------------------*/

/*  sig_handler()
 *
 *  Signal Action Handler function
 */

static void sig_handler( int signal )
{
  /* Wrap up and quit */
  Cleanup();
  fprintf( stderr, "\n" );
  switch( signal )
  {
	case SIGINT :
	  fprintf( stderr, "%s\n",  "xfhell: Exiting via User Interrupt" );
	  exit( signal );

	case SIGSEGV :
	  fprintf( stderr, "%s\n",  "xfhell: Segmentation Fault" );
	  exit( signal );

	case SIGFPE :
	  fprintf( stderr, "%s\n",  "xfhell: Floating Point Exception" );
	  exit( signal );

	case SIGABRT :
	  fprintf( stderr, "%s\n",  "xfhell: Abort Signal received" );
	  exit( signal );

	case SIGTERM :
	  fprintf( stderr, "%s\n",  "xfhell: Termination Request received" );
	  exit( signal );
  }

} /* End of sig_handler() */

/*------------------------------------------------------------------------*/

/* Functions for testing and setting/clearing flow control flags
 *
 *  See xfhell.h for definition of flow control flags
 */

/* An int variable holding the single-bit flags */
static int Flags = 0;

  int
isFlagSet( int flag )
{
  return( (Flags & flag) == flag );
}

  int
isFlagClear( int flag )
{
  return( (~Flags & flag) == flag );
}

  void
SetFlag( int flag )
{
  Flags |= flag;
}

  void
ClearFlag( int flag )
{
  Flags &= ~flag;
}

  void
ToggleFlag( int flag )
{
  Flags ^= flag;
}

/*------------------------------------------------------------------------*/

/*  Cleanup()
 *
 *  Cleans up before quitting
 */

  void
Cleanup( void )
{
  /* Save outstanding records */
  if( isFlagSet(SAVE_RECORD) )
	Save_QSO_Record();
	Save_Pixbuf();

  /* Close open files */
  if( rc_data.qso_record_fp != NULL )
  {
	ClearFlag(RECORD_QSO);
	fclose( rc_data.qso_record_fp );
	rc_data.qso_record_fp = NULL;
  }

  if( rc_data.log_adif_fp != NULL )
  {
	fclose( rc_data.log_adif_fp );
	rc_data.log_adif_fp = NULL;
  }

  if( rc_data.station_log_fp != NULL )
  {
	fclose( rc_data.station_log_fp );
	rc_data.station_log_fp = NULL;
  }

  /* Release sound card */
  if( rc_data.dsp_in_fd > 0 )
  {
	close( rc_data.dsp_in_fd );
	rc_data.dsp_in_fd = 0;
  }

  if( rc_data.dsp_out_fd > 0 )
  {
	close( rc_data.dsp_out_fd );
	rc_data.dsp_out_fd = 0;
  }

  if( rc_data.mix_fd > 0 )
  {
	close( rc_data.mix_fd );
	rc_data.mix_fd = 0;
  }

  /* Release sound card and Tcvr */
  ClearFlag
	(
	 SOUND_SETUP    |
	 TRANSMIT_MODE  |
	 TRANSMIT_MACRO |
	 TRANSMIT_KEYBD |
	 RECEIVE_MODE
	);

  Set_Tcvr_Mode();
  Close_Tcvr_Serial();

} /* Cleanup( void ) */

/*------------------------------------------------------------------------*/

